Title: Backup and Recovery - Ceph
Status: In Progress

# Backup and Recovery - Ceph

## Introduction

A snapshot is a read-only copy of the state of an image at a particular point in time. One of the 
advanced features of Ceph block devices is that the user can create image snapshots
to retain a history of image state. Ceph also supports snapshot layering, which allows 
images to be cloned (example: a VM image) quickly and easily. Ceph supports block device snapshots 
using the `rbd` command and many high-level interfaces to include OpenStack.

## Scope

**TODO**

## Backup

To create a snapshot with `rbd`, specify the `snap create` option, the pool name, and the 
image name.

```
rbd --pool {pool-name} snap create --snap {snap-name} {image-name}
rbd snap create {pool-name}/{image-name}@{snap-name}
```

For example:

```
rbd --pool rbd snap create --snap snapname foo
rbd snap create rbd/foo@snapname
```

## Restore

To rollback to a snapshot with `rbd`, specify the `snap rollback` option, the pool name, the 
image name and the snap name.

```
rbd --pool {pool-name} snap rollback --snap {snap-name} {image-name}
rbd snap rollback {pool-name}/{image-name}@{snap-name}
```

For example:

```
rbd --pool rbd snap rollback --snap snapname foo
rbd snap rollback rbd/foo@snapname
```

**Note:** Rolling back an image to a snapshot means overwriting the current version of the image 
with snapshot data. The time it takes to execute a rollback increases with the size of the 
image. It is faster to clone from a snapshot than to rollback an image to a snapshot. Cloning from a snapshot is the preferred method of returning to a pre-existing state.

## Maintenance

Taking snapshots increases your level of security, but also eats disk space. To delete older ones
you can list them, delete individual ones, or purge all snapshots.

To list snapshots of an image, specify the pool name and the image name.

```
rbd --pool {pool-name} snap ls {image-name}
rbd snap ls {pool-name}/{image-name}
```

For example:

```
rbd --pool rbd snap ls foo
rbd snap ls rbd/foo
```

To delete a snapshot with `rbd`, specify the `snap rm` option, the pool name, the image name 
and the username.

```
rbd --pool {pool-name} snap rm --snap {snap-name} {image-name}
rbd snap rm {pool-name}/{image-name}@{snap-name}
```

For example:

```
rbd --pool rbd snap rm --snap snapname foo
rbd snap rm rbd/foo@snapname
```

**Note:** Ceph OSDs delete data asynchronously, so deleting a snapshot doesn’t free up the 
disk space immediately.

To delete all snapshots for an image with `rbd`, specify the snap purge option and the 
image name.

```
rbd --pool {pool-name} snap purge {image-name}
rbd snap purge {pool-name}/{image-name}
```

For example:

```
rbd --pool rbd snap purge foo
rbd snap purge rbd/foo
```
