Title: Backup and Recovery - OpenStack
Status: In Progress - jen checking

# Backup and Recovery - OpenStack

## Introduction

The OpenStack flexibility makes backup and restore a very individual process,
depending on the used components. This section describes how the critical parts OpenStack 
needs to run, like configuration files, and databases, are saved. Just like 
before with Juju, it doesn't describe how to backup the objects inside the Object 
storage or the data inside the Block Storage.

## Scope

**TODO**

## Backup Cloud Controller Database

Like Juju, the OpenStack cloud controller uses a database server that stores the
central databases for Nova, Glance, Keystone, Cinder, and Swift. You can backup
the five databases into one common dump:

````
$ mysqldump --opt --all-databases > openstack.sql
````

Alternatively you can backup the database for each component individually:

````
$ mysqldump --opt nova > nova.sql
$ mysqldump --opt glance > glance.sql
$ mysqldump --opt keystone > keystone.sql
$ mysqldump --opt cinder > cinder.sql
$ mysqldump --opt swift > swift.sql
````

## Backup File Systems

In addition to the databases, OpenStack uses different directories for its configuration,
runtime files, and logging. Like the databases, they are grouped individually per
component. This also allows for the backup to be done per component.

### Nova

You'll find the configuration directory `/etc/nova` on the cloud controller and 
each compute node. It should be regularly backed up.

Another directory to backup is `/var/lib/nova`. Here, you must be careful
with the `instances` subdirectory on the compute nodes. It contains the KVM images
of the running instances. If you want to maintain backup copies of those instances,
you can do a backup here too. In this case, make sure not to save a live KVM instance
because it may not boot properly after restoring the backup.

Third directory for the compute component is `/var/log/nova`. In case of a central
logging server, this directory does not need to be backed up. We suggest you
run your environment with this kind of logging.

### Glance

Like with Nova, you'll find the directories `/etc/glance` and `/var/log/glance`. The
handling should be the same here as well.

Glance also uses the directory named `/var/lib/glance`, which should also be backed
up.

### Keystone

Keystone is using the directories `/etc/keystone`, `/var/lib/keystone`, and
`/var/log/keystone`. They follow the same rules as Nova and Glance. Even if
the `lib` directory doesn't contain any data being used, it can also be backed 
up just in case.

### Cinder

As before, you'll find the directories `/etc/cinder`, `/var/log/cinder`,
and `/var/lib/cinder`. The handling here should also be the same. Unlike Nova and Glance, there's no special handling of `/var/lib/cinder` needed.

### Swift

Beside the Swift configuration, the directory `/etc/swift` contains ring files
and ring builder files. If those get lost, your data becomes inaccessable.
You can imagine how important it is to backup this directory. Best practise
is to copy the builder files to the storage nodes along with the ring files. This ensures that
multiple copies are spread throughout the cluster.

**TODO(mue)** Really needed when we use Ceph for storage?

## Restore

The restore based on the backups is a step-by-step process restoring the components
databases and all related directories. It's essential the component to restore is not running. Always start the restoring after you've stopped all components.

Take Nova for example. First execute:

````
$ stop nova-api
$ stop nova-cert
$ stop nova-consoleauth
$ stop nova-novncproxy
$ stop nova-objectstore
$ stop nova-scheduler
````

on the cloud controller to safely stop the processes of the component. Next step is to
restore the database. By using the `--opt` option during backup, we ensured all
tables are initially dropped and there's no conflict with existing data in
the databases.

````
$ mysql nova < nova.sql
````

Before restoring the directories, you should move the configuration directory `/etc/nova` into a secure location in case you need to roll it back.

After the database and the files are restored, you can start MySQL and Nova again.

````
$ start mysql
$ start nova-api
$ start nova-cert
$ start nova-consoleauth
$ start nova-novncproxy
$ start nova-objectstore
$ start nova-scheduler
````

The process for the other components look similar.
