Title: Scaling - OpenStack
Status: In Progress

# Scaling - OpenStack

## Introduction

While traditional applications require larger hardware to scale ("vertical scaling"), 
cloud-based applications typically request more, discrete hardware ("horizontal scaling"). 
If your cloud is successful, eventually you must add resources to meet the increasing 
demand.

To suit the cloud paradigm, OpenStack is designed to be horizontally scalable. 
Rather than switching to larger servers, you procure more servers and simply install 
identically configured services. Ideally, you scale out and load balance among groups 
of functionally identical services.

But to scale the services running on OpenStack, sometimes even OpenStack itself has to
be scaled. This means nodes for computing or storage have to be added.

## Nova

### Usage Statistics

To list the hosts and the nova-related services that run on them call:

```
nova host-list

+---------------+-------------+----------+
| host_name     | service     | zone     |
+---------------+-------------+----------+
| mystack-alpha | conductor   | internal |
| mystack-alpha | compute     | nova     |
| mystack-alpha | cert        | internal |
| mystack-alpha | network     | internal |
| mystack-alpha | scheduler   | internal |
| mystack-alpha | consoleauth | internal |
+---------------+-------------+----------+

```

To get a summary of resource usage of all of the instances running on the host call:

```
nova host-describe mystack-alpha

+---------------+----------------------------------+-----+-----------+---------+
| HOST          | PROJECT                          | cpu | memory_mb | disk_gb |
+---------------+----------------------------------+-----+-----------+---------+
| mystack-alpha | (total)                          | 2   | 4003      | 157     |
| mystack-alpha | (used_now)                       | 3   | 5120      | 40      |
| mystack-alpha | (used_max)                       | 3   | 4608      | 40      |
| mystack-alpha | b70d90d65e464582b6b2161cf3603ced | 1   | 512       | 0       |
| mystack-alpha | 66265572db174a7aa66eba661f58eb9e | 2   | 4096      | 40      |
+---------------+----------------------------------+-----+-----------+---------+

```

This information will help explain how this host is used.

- The `cpu` column shows the sum of the virtual CPUs for instances running on the host.
- The `memory_mb` column shows the sum of the memory (in MB) allocated to the instances 
  that run on the hosts.
- The `disk_gb` column shows the sum of the root and ephemeral disk sizes (in GB) of 
  the instances that run on the hosts.
- The `used_now` row shows the sum of the resources allocated to the instances that run 
  on the host plus the resources allocated to the virtual machine of the host itself.
- The `used_max` row shows the sum of the resources allocated to the instances that run 
  on the host.

**Note:** These values are computed using only information about flavors of the 
instances that run on the hosts. This command does not query the CPU usage, memory usage, 
or hard disk usage of the physical host.


Now you can retrieve the CPU, memory, I/O, and network statistics for an instance. First
list all instances.


```
nova list

+--------------------------------------+----------------------+--------+------------+-------------+------------------+
| ID                                   | Name                 | Status | Task State | Power State | Networks         |
+--------------------------------------+----------------------+--------+------------+-------------+------------------+
| 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5 | myCirrosServer       | ACTIVE | None       | Running     | private=10.0.0.3 |
| 8a99547e-7385-4ad1-ae50-4ecfaaad5f42 | myInstanceFromVolume | ACTIVE | None       | Running     | private=10.0.0.4 |
+--------------------------------------+----------------------+--------+------------+-------------+------------------+
```

Then, get some diagnostic statistics:

```
nova diagnostics myCirrosServer

+------------------+----------------+
| Property         | Value          |
+------------------+----------------+
| vnet1_rx         | 1210744        |
| cpu0_time        | 19624610000000 |
| vda_read         | 0              |
| vda_write        | 0              |
| vda_write_req    | 0              |
| vnet1_tx         | 863734         |
| vnet1_tx_errors  | 0              |
| vnet1_rx_drop    | 0              |
| vnet1_tx_packets | 3855           |
| vnet1_tx_drop    | 0              |
| vnet1_rx_errors  | 0              |
| memory           | 2097152        |
| vnet1_rx_packets | 5485           |
| vda_read_req     | 0              |
| vda_errors       | -1             |
+------------------+----------------+
```

Finally you can get summary statistics for each tenant:

```
nova usage-list

Usage from 2013-06-25 to 2013-07-24:
+----------------------------------+-----------+--------------+-----------+---------------+
| Tenant ID                        | Instances | RAM MB-Hours | CPU Hours | Disk GB-Hours |
+----------------------------------+-----------+--------------+-----------+---------------+
| b70d90d65e464582b6b2161cf3603ced | 1         | 344064.44    | 672.00    | 0.00          |
| 66265572db174a7aa66eba661f58eb9e | 3         | 671626.76    | 327.94    | 6558.86       |
+----------------------------------+-----------+--------------+-----------+---------------+
```

### Change server size

In case an image size doesn't match your needs anymore you're able to resize it. First
list your available flavors.

```
$ nova flavor-list
+----+-----------+-----------+------+-----------+------+-------+-------------+
| ID |    Name   | Memory_MB | Disk | Ephemeral | Swap | VCPUs | RXTX_Factor |
+----+-----------+-----------+------+-----------+------+-------+-------------+
| 1  |  m1.tiny  |    512    |  0   |     0     |      |   1   |     1.0     |
| 2  |  m1.small |    2048   |  10  |     20    |      |   1   |     1.0     |
| 3  | m1.medium |    4096   |  10  |     40    |      |   2   |     1.0     |
| 4  |  m1.large |    8192   |  10  |     80    |      |   4   |     1.0     |
| 5  | m1.xlarge |   16384   |  10  |    160    |      |   8   |     1.0     |
+----+-----------+-----------+------+-----------+------+-------+-------------+
```

Now you can show the information for the server you want to resize with:

```
$ nova show 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5
+------------------------+----------------------------------------------------------+
| Property               | Value                                                    |
+------------------------+----------------------------------------------------------+
| OS-DCF:diskConfig      | MANUAL                                                   |
| OS-EXT-STS:power_state | 1                                                        |
| OS-EXT-STS:task_state  | None                                                     |
| OS-EXT-STS:vm_state    | active                                                   |
| OS-EXT-SRV-ATTR:host   | mystack-alpha                                            |
| accessIPv4             |                                                          |
| accessIPv6             |                                                          |
| config_drive           |                                                          |
| created                | 2012-05-09T15:47:48Z                                     |
| flavor                 | m1.small                                                 |
| hostId                 | de0c201e62be88c61aeb52f51d91e147acf6cf2012bb57892e528487 |
| id                     | 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5                     |
| image                  | maverick-image                                           |
| key_name               |                                                          |
| metadata               | {}                                                       |
| name                   | myCirrosServer                                           |
| private network        | 172.16.101.6                                             |
| progress               | 0                                                        |
| public network         | 10.4.113.6                                               |
| status                 | ACTIVE                                                   |
| tenant_id              | e830c2fbb7aa4586adf16d61c9b7e482                         |
| updated                | 2012-05-09T15:47:59Z                                     |
| user_id                | de3f4e99637743c7b6d27faca4b800a9                         |
+------------------------+----------------------------------------------------------+
```

Here in the example the flavor is `m1.small`. To resize it to `m1.medium` you need
the server's ID and the ID of the flavor:

```
$ nova resize 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5 3
```

Now the `list` command shows the changed status:

```
$ nova list
+--------------------------------------+----------------------+--------+------------+-------------+------------------+
| ID                                   | Name                 | Status | Task State | Power State | Networks         |
+--------------------------------------+----------------------+--------+------------+-------------+------------------+
| 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5 | myCirrosServer       | RESIZE | None       | Running     | private=10.0.0.3 |
| 8a99547e-7385-4ad1-ae50-4ecfaaad5f42 | myInstanceFromVolume | ACTIVE | None       | Running     | private=10.0.0.4 |
+--------------------------------------+----------------------+--------+------------+-------------+------------------+
```

Once this operation finished the status changes to `VERIFY_RESIZE`. You have to confirm
that this operation has been successful:

```
$ nova resize-confirm 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5
```

In case the resizing hasn't been successful you can revert it by executing:

```
$ nova resize-revert 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5
```

In both cases, the server status should go back to `ACTIVE`.

### Adding compute nodes

Adding compute nodes is straightforward. They are easily picked up by the existing 
installation. Simply call:

```
juju add-unit nova-compute
```

### Migrate instances to other nodes

OpenStack allows to migrate instances between nodes, e.g. in case of an overall 
too high load on the current node. You alreade know the commands `list` and `show`
to retrieve information about your instance. To list the possible nodes 
execute:

```
nova-manager service list
mystack-alpha nova-scheduler enabled  :-) None
mystack-alpha nova-network enabled  :-) None
mystack-alpha nova-compute enabled  :-) None
mystack-bravo nova-compute enabled  :-) None
mystack-charlie nova-compute enabled  :-) None
```

Here we will choose `mystack-charlie` as migration target, `nova-compute`
is running on this system. First let's have a look if the node has
enough resources:

```
nova-manage service describe_resource mystack-charlie

HOST                       PROJECT     cpu   mem(mb)     hdd
mystack-charlie(total)                  16     32232     878
mystack-charlie(used_now)               13     21284     442
mystack-charlie(used_max)               13     21284     442
mystack-charlie            p1            5     10240     150
mystack-charlie            p2            5     10240     150
.....
```

These values show the physical resources and their usage.

- The `cpu` column shows the number of cpu.
- The `mem(mb)` column is the total amount of memory (MB).
- The `hdd' column shows the total amount of space for NOVA-INST-DIR/instances (GB).
- 1st line shows total amount of resource physical server has.
- 2nd line shows current used resource.
- 3rd line shows maximum used resource.
- 4th line and under is used resource per project.

If those values show that our instance fits on that system we can
migrate it with:

```
nova live-migration 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5 mystack-charlie

Migration of 84c6e57d-a6b1-44b6-81eb-fcb36afd31b5 initiated.
```

## Cinder

**TODO**
