Title:Installing Juju

#Installing Juju for Ubuntu Cloud

![logo](../media/juju_logo.png)

##Introduction 
Juju is a powerful tool for managing scale out architectures in the cloud. It bootstraps an instance in your cloud from where it can deploy, relate, manage and scale services in all directions. Running from the commandline or an intuitive GUI, it delivers on its promise to _orchestrate_ services rather than simply deploy them

In modern scale out architectures, servers are just units that enable
application services to scale. Services are managed independently of the
underlying hardware so you don’t need to worry about launching new
instances and setting up config files to connect applications, Juju just
takes care of it all.

Other solutions focus on configuration management to enforce consistency
across scale out architectures but Juju creates services as building
blocks that are connected together simply by drawing a line between the
two. It is this service based approach that allows DevOps and architects
to quickly be able to visualise, design, deploy and scale their
application infrastructures far more easily than if they are stuck in
the weeds with configuration management tools. For more background on
Juju, please [see the main Juju website][juju]

##Scope of this documentation 
The Juju client can run on a variety of architectures, and can be easily 
configured to work with different cloud providers - or no cloud provider at all, 
creating all instances on a single machine. The purpose of this document is to 
enable users to install and configure Juju to work as part of an Ubuntu Cloud 
OpenStack deployment.

##Assumptions

To provide clarity in the instructions for the specific purposes
outlined above, this document makes the following assumptions.

 * You are installing Juju in conjunction with a MAAS deployment.
 * The Juju client will be running on, or can communicate with the MAAS controller node.
 * The Juju client will be installed on the Ubuntu 14.04 LTS release.

##Installing Juju 
To install Juju, use the standard package installer:

```
sudo apt-get update
sudo apt-get install juju-core
```

There are some additional tools which will be useful for testing your 
configuration and working with charms which should also be installed:

```
sudo apt-get install juju-quickstart juju-deployer charm-tools
```

##Configuring Juju to work with MAAS
Now the Juju software is installed, it needs to be configured to work
with MAAS. This is done as follows.

###1. Generate an SSH key

MAAS and Juju both use SSH to authenticate access to running nodes. 
If you do not have an SSH key, you should generate one with the following
command:

```
ssh-keygen -t rsa -b 2048
```

This key will be associated with the user account you are running in at the 
time, so should be the same account you intend to run Juju from.


###2. Obtain the MAAS API/OAuth key 
You’ll need an API key from MAAS so that the Juju client can access it. Each 
user account in MAAS can have as many API keys as desired but you should 
use a different user account for each Juju environment you want to use within
MAAS.

In a web browser, navigate to the MAAS home page (this will be at
`http://{address}:80/MAAS/` where {address} is the IP address or
hostname of the MAAS controller node). Go to your MAAS preferences page
(click your username at the top-right of the page).

MAAS will have already generated a key to use (though you may add others
for more environments if you wish) displayed as indicated in the image
below.

![maas preferences](../media/maas-preferences.png)

Make sure you copy the entire key. It is quite long and doesn't fully
fit in the text box of the web interface. Paste this key somewhere for
reference - you will need it for the next step.

###3. Edit the Juju environments.yaml file 
This is done by generating and editing a file, environments.yaml, which will
live in your ~/.juju/ directory. You can generate the environments file manually
, but Juju also includes a boilerplate configuration option that will flesh out
most of the file for you and minimise the amount of work (and potential
errors).

To generate an initial config file, you simply need to run: 

```
juju generate-config
```

This command will cause a template file to be written to your ~/.juju directory
if an environments.yaml file does not already exist. It will also create
the ~./juju directory if that does not exist.

If you have previously installed Juju on this machine and the
environments.yaml file already exists, you should simply edit
it for your new MAAS configuration.

This file will contain sample profiles for different types of cloud
services, but you will need only need to edit the section labelled `maas`,
which should look something like this :

```
 maas:
    type: maas
    # Change this to where your MAAS server lives.  It must specify the base path.
    maas-server: 'http://192.168.1.1/MAAS/'
    maas-oauth: '<add your OAuth credentials from MAAS here>'
    # default-series: precise
    authorized-keys-path: ~/.ssh/authorized_keys # or any file you want.
    # Or:
    # authorized-keys: ssh-rsa keymaterialhere
```

The only required information here are the type, maas-server, maas-oauth and one 
or other of the ways of specifying your ssh keys. The default-series itself 
defaults to 'trusty' and MAAS will add your SSH keys. So your config should be 
edited to include the correct address for the MAAS master node and the key which 
you collected in the previous step:

```
 maas:
    type: maas
    maas-server: 'http://172.16.100.1/MAAS/'
    maas-oauth: '--MAAS API key string--'
    authorized-keys-path: ~/.ssh/authorized_keys
```

You may optionally edit the line near the top of the file which reads:

```
 default: amazon 
``` 

to read:

```
 default: maas 
``` 

..and skip step 4 below.

!!! note If you wish to configure your Juju client to additionally work
with other cloud environments, please see the [documentation on the main
Juju website][juju-docs].

###4. Select the MAAS environment 
To select the MAAS environment from your configuration file, run the following 
Juju command: 

``` 
juju switch maas 
```

Any further Juju commands will now apply to that specific environment.
The switch setting is persistent, so it will remain applicable up until
you issue another switch command. You can check which environment is
selected by running the command without additional arguments:

```
juju switch 
```

##Environment testing

Once Juju has been installed and configured to use MAAS, it is useful to
test that it is operating properly. The easiest way to do this is to run
the Juju quickstart command, which will create a Juju bootstrap node and
then deploy the juju-gui charm on it. 

```
juju quickstart
```

The juju-gui web interface will then be automatically loaded in the default web
browser. This process may take several minutes to complete.

Your browser may come up with a security warning such as this:

![Security warning](../media/juju-security.png)

This is because the web-server doesn't have security credentials (it was
just created after all). You should accept any options which allow you
to proceed and view the site. Juju will automatically log you in to the
juju-gui interface, and you should see a screen like this:

![juju-gui.png](../media/juju-gui.png)

Once you are satisfied that Juju is working correctly, you can destroy
this running environment: 
```
juju destroy-environment maas
```

You will need to confirm this action by typing 'y' at the prompt.

This will remove the bootstrap and juju-gui instance and return the used
node back to the pool in MAAS.

[juju]:(http:///juju.ubuntu.com) 
[juju-docs]:(http:///juju.ubuntu.com/docs)


