Title: Landscape
#Managing OpenStack with Landscape

##About Landscape
Landscape is a system management tool designed to let you easily manage multiple Ubuntu systems - up to 40,000 with a single Landscape instance. From a single dashboard you can apply package updates and perform other administrative tasks on many machines. You can categorize machines by group, and manage each group separately. You can make changes to targeted machines even when they are offline; the changes will be applied next time they start. Landscape lets you create scripts to automate routine work such as starting and stopping services and performing backups. It lets you use both common Ubuntu repositories and any custom repositories you may create for your own computers. Landscape is particularly adept at security updates; it can highlight newly available packages that involve security fixes so they can be applied quickly. You can use Landscape as a hosted service as part of Ubuntu Advantage, or run it on premises via Landscape Dedicated Server.

##Ubuntu Advantage
Ubuntu Advantage comprises systems management tools, technical support, access to online resources and support engineers, training, and legal assurance to keep organizations on top of their Ubuntu server, desktop, and cloud deployments. Advantage provides subscriptions at various support levels to help organizations maintain the level of support they need.





##Access groups




Landscape lets administrators limit administrative rights on computers
by assigning them to logical groupings called access groups. Each
computer can be in only one access group, but you can organize access
groups hierarchically to mirror the organization of your business. In
addition to computers, access groups can contain package profiles,
scripts, and custom graphs.

Creating access groups
----------------------

A new Landscape installation comes with a single access group, called
global, which gives any administrators who are associated with roles
that include that access group control over every computer managed by
Landscape. Most organizations will want to subdivide administration
responsibilities by creating logical groupings of computers. You can
create new access groups from the ACCESS GROUPS menu under your account
menu.

**Figure 5.1.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups1.png)

\

To create a new access group, you must provide two pieces of
information: a title for the access group and a parent.

To start with, the parent must be the global access group. If you want a
flat management hierarchy, you can make every access group a child of
global. Alternatively, you can use parent/child relationships to create
a hierarchy of access groups. For instance, you could specify different
sites at a high level, and under them individual buildings, and finally
individual departments. Such a hierarchy allows you to specify groups of
computers to be managed together by one administrator. Administrators
whose roles are associated with higher-level access groups can manage
all subgroups of which their access group is a parent.

When a new access group is first created, its administrators are those
who have roles linked to its parent access group, but you can edit the
roles associated with an access group. To change the roles associated
with an access group, see
[below](https://landscape.canonical.com/static/doc/user-guide/ch05.html#associatingadmins "Associating roles with access groups").

Adding computers to access groups
---------------------------------

To see all the computers currently in an access group, click on the name
of the group in the ACCESS GROUPS screen. The screen that then appears
displays information about that group. On the right side of the screen,
click the word "computers" to show the list of computers that are
currently members of this access group.

**Figure 5.2.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups2.png)

\
Alternatively, you can click on the COMPUTERS menu item at the top of
the Landscape screen, and in the selection box at the top of the left
column, enter `access-group:`{.literal} followed by the name of your
access group: for instance, `access-group:stagingservers`{.literal}.

To add computers to an access group, click on the COMPUTERS menu item at
the top of the Landscape screen. The resulting INFO screen shows the
total number of available computers being managed by Landscape, and the
number of computers currently selected:

**Figure 5.3.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups3.png)

\
Find computers you wish to include (see the documentation on [selecting
computers](https://landscape.canonical.com/static/doc/user-guide/ch06.html#selectingcomputers "Selecting computers")),
then tick the checkbox next to each computer you wish to select. Once
you've made your selection, click on the INFO menu entry at the top of
the page Scroll down to the bottom section, choose the access group you
want from the drop-down list, then click Update access group.

**Figure 5.4.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups4.png)

\

Associating roles with access groups
------------------------------------

An administrator may manage an access group if he is associated with a
role that has permission to do so. To associate a role with one or more
access groups, click on the ROLES menu item under your account to
display a screen that shows a role membership matrix.

**Figure 5.5.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups5.png)

\
The top of that screen shows a list of role names. Click on a role name
to edit the permissions and access groups associated with that role.
Note that you cannot modify the GlobalAdmin role, so there is no link
associated with that label at the top of the matrix.

Editing access groups
---------------------

To change the name or title of an existing access group, click on the
name of the group in the ACCESS GROUPS screen, then click on the Edit
access group link at the top of next screen. Make changes, then click
Save.

**Figure 5.6.**

![image](./Chapter%A05.%A0Access%20groups_files/accessgroups6.png)

\

Deleting access groups
----------------------

To delete an existing access group, click on the name of the group in
the ACCESS GROUPS screen, then click on the Edit access group link at
the top of next screen. On the resulting screen, click the Delete
button. You may Confirm the group's deletion, or you can click Cancel to
abort the operation. When you delete an access group, its resources move
to its parent access group.

**Figure 5.7.**



##Managing computers


Provisioning new computers
--------------------------

Landscape can provision computers in two ways: manually, or via metal as
a service (MAAS). [The Ubuntu wiki explains how to set up
MAAS](https://wiki.ubuntu.com/ServerTeam/MAAS/).

To manually provision computers, click on PROVISIONING under your
ACCOUNT menu. Landscape displays a provisioning dashboard that shows the
number of provisioning servers you have set up, managed systems, and
pending systems.

**Figure 6.1.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers1.png)

\

To provision new systems, click the Provision new systems link. On the
Provisioning New Systems screen, the top three fields apply to all the
computers you wish to provision at one time.

**Figure 6.2.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers2.png)

\
Enter the Ubuntu release/architecture from a drop-down list; the
available choices are the two hardware architectures (i386 and amd64)
for the each Ubuntu release beginning with 12.04. Enter the access group
to which the new systems should belong from a drop-down list of the
access groups set up for your account. You can optionally enter user
data, which Landscape can use for special processing. For instance, you
could use this field with Ubuntu's
[cloud-init](https://help.ubuntu.com/community/CloudInit) utility, which
handles early initialization functions for a cloud instance.

For each computer you wish to provision, enter its MAC address,
hostname, an optional title that will be displayed on the computer
listing screen after the computer is set up, and optional tags separated
by commas that can later help you search for this computer. Click the
Add more systems link to get a new line of empty boxes into which you
can add data.

When you click the Next button, Landscape displays a screen that lets
you review the information you entered.

**Figure 6.3.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers3.png)

\
You can click on Back to make changes, or Provision to perform the
operation. Landscape then displays a status screen that at first shows
the specified computers waiting to boot on the MAAS server.

**Figure 6.4.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers4.png)

\

Registering computers
---------------------

If a computer is provisioned by Landscape, is it automatically
registered with Landscape, but when you first install Landscape, your
computers are not known to the Landscape server. To manage them, you
must register them with the server. Complete instructions for
registering client computers with a Landscape server are available at
https://yourserver/standalone/how-to-register. You can get to this page
by first clicking on the menu item for your account page on the top
menu, then on the link in the box on the left side of the page.

Selecting computers
-------------------

You can select one or more computers individually, or by using searches
or tags. For each of those approaches, the starting place is the
COMPUTERS menu entry at the top of the screen. Clicking on it displays a
list of all computers Landscape knows about.

-   To select computers individually, tick the boxes beside their names
    in the Select computers list.

-   Using searches - The upper left corner of the Select computers
    screen displays the instructions "Refine your selection by searching
    or selecting from the tags below," followed by a search box. You can
    enter any string in that box and press Enter, or click the arrow
    next to the box. Landscape will search both the name and hostname
    associated with all computers for a match with the search term.
    Searches are not case-sensitive. A list of matching computers is
    displayed on the right side of the screen.

    Once you've selected a group of computers, you can apply a tag to
    them to make it easier to find them again. To do so, with your
    computers selected, click on INFO under COMPUTERS. In the box under
    Tags:, enter the tag you want to use and click Add.

-   Using tags - Any tags you have already created appear in a list
    under the search box on the left of the Computers screens. You can
    click on any tag to display the list of computers associated with
    it. To select any of the displayed computers, tick the box next to
    its name, or click Select: All link at the top of the list.

Information about computers
---------------------------

By clicking on several submenus of the COMPUTERS menu, you can get
information about selected computers.

-   Clicking on ACTIVITIES displays information about actions that may
    be applied to computers. You can filter the activity log to show
    All, Pending, Unapproved, or Failed activities. You can click on
    each activity in the list to display a screen showing details about
    the activity. On that screen you can Approve, Cancel, Undo, or Redo
    the activity by clicking on the relevant button.

-   Clicking on HARDWARE displays information about the selected
    computer's processor, memory, network, storage, audio, video, PCI,
    and USB hardware, as well as BIOS information and CPU flags.

    **Figure 6.5.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers5.png)

    \

-   Clicking on PROCESSES displays information about all processes
    running on a computer at the last time it checked in with the
    Landscape server, and lets you end or kill processes by selecting
    them and clicking on the relevant buttons.

    **Figure 6.6.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers6.png)

    \

-   Clicking on REPORTS displays seven pie charts that show what
    percentage of computers:

    -   are securely patched

    -   are covered by upgrade profiles

    -   have contacted the server within the last five minutes

    -   have applied security updates - four charts show computers that
        have applied Ubuntu Security Notices within the last two, 14,
        30, and 60+ days

-   Clicking on MONITORING displays graphs of key performance
    statistics, such as CPU load, memory use, disk use, and network
    traffic.

    **Figure 6.7.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers7.png)

    \
    You can also create custom graphs to display at the top of the page
    by clicking on the Create some now! link. A drop-down box at the top
    of the page lets you specify the timeframe the graph data covers:
    one day, three days, one week, or four weeks. You can download the
    data behind each graph by clicking the relevant button under the
    graph.

The activity log
----------------

The right side of the dashboard that displays when you click on your
account menu, and when you click on the ACTIVITIES submenu, shows the
status of Landscape activities, displayed in reverse chronological
order.

**Figure 6.8.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers8.png)

\
You can view details on an individual activity by clicking on its
description. Each activity is labeled with a status; possible values
are:

-   Succeeded

-   In progress

-   Scheduled

-   Queued

-   Unapproved

-   Canceled

-   Failed

You can select a subset to view by clicking on the links above the table
for All, Pending, Unapproved, or Failed activities.

In addition to the status and description of each activity, the table
shows what computers the activity applied to, who created it, and when.

Managing users
--------------

Clicking on USERS displays a list of users on each of the selected
computers.

**Figure 6.9.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers9.png)

\
You can select one or more users, then click one of the buttons at the
top of the screen:

-   The ADD button lets you add a new user to the selected computers.

    **Figure 6.10.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers10.png)

    \
    You must specify the person's name, a username, and a passphrase.
    You may also specify a location and telephone numbers. Click the ADD
    button at the bottom of the screen to complete the operation.

-   The DELETE button displays a screen that lets you delete the
    selected users.

    **Figure 6.11.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers11.png)

    \
    You may also tick a checkbox to delete the user's home folders as
    well. Press the Delete button at the bottom of the screen to
    complete the operation.

-   The EDIT button displays a User details screen that lets you change
    details such as the person's name, primary group, passphrase,
    location, and telephone numbers, and add or remove the user from
    groups on the selected computers.

    **Figure 6.12.**

    ![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers12.png)

    \

-   The LOCK button prevents the selected users from logging into their
    accounts.

-   The UNLOCK button lets users into their cars when they've
    accidentally locked their keys inside. Actually, no, it simply
    unlocks previously locked accounts.

Managing alerts
---------------

Landscape uses alerts to notify administrators of conditions that
require attention. The following types of alerts are available:

-   when a pending computer needs to be accepted or rejected

-   when you are exceeding your license entitlements for Landscape
    Dedicated Server (This alert does not apply to the hosted version of
    Landscape.)

-   when new package updates are available for computers

-   when new security updates are available for computers

-   when a package profile is not applied

-   when package reporting fails (Each client runs the command **apt-get
    update** every 60 minutes. Anything that prevents that command from
    succeeding is considered a package reporting failure.)

-   when an activity requires explicit administrator acceptance or
    rejection

-   when a computer has not contacted the Landscape server for more than
    five minutes

-   when computers need to be rebooted in order for a package update
    (such as a kernel update) to take effect

To configure alerts, click on the Configure alerts link in the
dashboard, or click on your account's ALERTS menu item. Tick the check
box next to each type of alert you want to subscribe to, or click the
All or None buttons at the top of the table, then click on the Subscribe
or Unsubscribe button below the table.

**Figure 6.13.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers13.png)

\

The Alerts screen shows the status of each alert. If an alert has not
been tripped, the status is OK; if it has, the status is Alerted. The
last column notes whether the alert applies to your account (pending
computers, for instance, are not yet Landscape clients, but they are
part of your account), to all computers, or to a specified set of tagged
computers.

If an alert is tripped, chances are an administrator should investigate
it. You can see alerts on the account dashboard that displays when you
click on your account name on the top menu. The description for each
alert is a link; click on it to see a table of alerts. When you click on
an alert, the resulting screen shows relevant information about the
problem. For instance, if you click on an alert about computers having
issues reporting packages, the table shows the computer affected, the
error code, and error output text.

**Figure 6.14.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers14.png)

\
On some alert screens you can download the list of affected computers as
a CSV file or save the criteria that generated the alert as a saved
search by clicking the relevant button at the bottom of the screen.

Managing scripts
----------------

Landscape lets you run scripts on the computers you manage in your
account. The scripts may be in any language, as long as an interpreter
for that language is present on the computers on which they are to run.
You can maintain a library of scripts for common tasks. You can manage
scripts from the STORED SCRIPTS menu under your account, and run them
against computers from the SCRIPTS menu under COMPUTERS.

The Stored scripts screen displays a list of existing scripts, along
with the access groups each belongs to and its creator.

**Figure 6.15.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers15.png)

\
You can edit a script by clicking on its name. To delete a stored
script, tick the check box next to its name, then click Remove. If you
have the proper permissions, Landscape erases the script immediately
without asking for confirmation.

From the Stored scripts screen you can add a new script by clicking on
Add stored script.

**Figure 6.16.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers16.png)

\
On the Create script screen you must enter a title, interpreter, the
script code, the time within which the script must complete, and the
access group to which the script belongs. You may enter a default user
to run the script as; if you don't, you will have to specify the user
when you choose to run the script. You may also attach as many as five
files with a maximum of 1MB in total size. On each computer on which a
script runs, attachments are placed in the directory specified by the
environment variable LANDSCAPE\_ATTACHMENTS, and are deleted once the
script has been run. After specifying all the information for a stored
script, click on Save to save it.

To run a stored script, go to the SCRIPTS menu under COMPUTERS. Here you
can choose to run a stored script, or run a new script.

**Figure 6.17.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers17.png)

\
When you choose to run an existing script, Landscape displays the script
details, which allows you to modify any information. You must specify
the user on the target computers to run the script as, and schedule the
script to run either as soon as possible, or at a specified time. When
you're ready to run the script, click on Run.

To run a new script, you must enter most of the same information you
would if you were creating a stored script, with three differences.

**Figure 6.18.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers18.png)

\
On this screen you must specify the user on the target computers to run
the script as, and you may optionally tick a check box to store the
script in your script library. You must also schedule the script to run
either as soon as possible, or at a specified time. When you're ready to
run the script, click on Run.

Managing upgrade profiles
-------------------------

An upgrade profile defines a schedule for the times when upgrades are to
be automatically installed on the machines associated with a specific
access group. You can associate zero or more computers with each upgrade
profile via tags to install packages on those computers. You can also
associate an upgrade profile with an access group, which limits its use
to only computers within the specified access group. You can manage
upgrade profiles from the UPGRADE PROFILES link in the PROFILES choice
under your account.

When you do so, Landscape displays a list of the names and descriptions
of existing upgrade profiles.

**Figure 6.19.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers19.png)

\
To see the details of an existing profile, click on its name to display
a screen that shows the name, schedule, and tags of computers associated
with the upgrade profile. If you want to change the upgrade profile's
name or schedule, click on the Edit upgrade profile link. If you want to
change the computers associated with the upgrade profile, tick or untick
the check boxes next to the tags on the lower part of the screen, then
click on the Change button. Though you can see the access group
associated with the upgrade profile, you cannot change the access groups
anywhere but from their association with a computer.

To add an upgrade profile, click on the Add upgrade profile link.

**Figure 6.20.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers20.png)

\
On the resulting Create an upgrade profile screen you must enter a name
for the upgrade profile. Names can contain only letters, numbers, and
hyphens. You may check a box to make the upgrade profile apply only to
security upgrades; if you leave it unchecked, it will target all
upgrades. Specify the access group to which the upgrade profile belongs
from a drop-down list. Finally, specify the schedule on which the
upgrade profile can run. You can specify a number of hours to let the
upgrade profile run; if it does not complete successfully in that time,
Landscape will trigger an alert. Click on the Save button to save the
new upgrade profile.

To delete one or more upgrade profiles, tick a check box next to the
upgrade profiles' names, then click on the Remove button.

Managing removal profiles
-------------------------

A removal profile defines a maximum number of days that a computer can
go without exchanging data with the Landscape server before it is
automatically removed. If more days pass than the profile's "Days
without exchange", that computer will automatically be removed and the
license seat it held will be released. This helps Landscape keep license
seats open and ensure Landscape is not tracking stale or retired
computer data for long periods of time. You can associate zero or more
computers with each removal profile via tags to ensure those computers
are governed by this removal profile. You can also associate a removal
profile with an access group, which limits its use to only computers
within the specified access group. You can manage removal profiles from
the REMOVAL PROFILES link in the PROFILES choice under your account.

When you do so, Landscape displays a list of the names and descriptions
of existing removal profiles.

**Figure 6.21.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers21.png)

\
To see the details of an existing profile, click on its name to display
a screen that shows the title, name and number of days without exchange
before the computer is automatically removed, and tags of computers
associated with the removal profile. If you want to change the removal
profile's title or number of days before removal, click on the Edit
removal profile link. If you want to change the computers associated
with the removal profile, tick or untick the check boxes next to the
tags on the lower part of the screen, then click on the Change button.
Though you can see the access group associated with the removal profile,
you cannot change the access groups anywhere but from their association
with a computer.

To add a removal profile, click on the Add removal profile link.

**Figure 6.22.**

![image](./Chapter%A06.%A0Managing%20computers_files/managecomputers22.png)

\
On the resulting Create a removal profile screen you must enter a title
for the removal profile. Specify the access group to which the removal
profile belongs from a drop-down list. Finally, specify the number of
days without exchange that computers will be allowed without contact
before they are automatically removed and their license seat is
released. If a computer does not contact Landscape within that number of
days, it will subsequently be removed. Click on the Save button to save
the new removal profile.

To delete one or more removal profiles, tick a check box next to the
removal profiles' names, then click on the Remove button.

##Managing packages


A package is a group of related files that comprise everything you need
to install an application. Packages are stored in repositories, and each
package is managed via a package profile, which is a record of the
package's dependencies and conflicts.

Package information
-------------------

Clicking on PACKAGES under the COMPUTERS menu displays a screen where
you can search for information about all the packages Landscape knows
about. You may first specify a package name or other search string, then
press Enter or click on the arrow next to the box. Landscape then
displays a list of packages that meet the search criteria.

**Figure 7.1.**

![image](./Chapter%A07.%A0Managing%20packages_files/managepackages1.png)

\
The top of the screen displays summary information about the packages:
clickable links to which computers have security updates and other
upgrades to be installed, and the number of computers that are
up-to-date and those that have not reported package information.

The next section provides a list of security issues on computers that
need security updates. You can click on the name or USN number of a
security issue to see a full Ubuntu Security Notice.

**Figure 7.2.**

![image](./Chapter%A07.%A0Managing%20packages_files/managepackages2.png)

\
The third section displays package information in the form of four
numbers for each selected computer: the number of packages available and
installed, pending upgrades, and held upgrades. You can click on the
number of pending or held upgrades to see a screen that lets you modify
the relevant package list and set a time for the upgrades to take place:

**Figure 7.3.**

![image](./Chapter%A07.%A0Managing%20packages_files/managepackages3.png)


Finally, a Request upgrades button at the bottom of the screen lets you
quickly request that all possible upgrades be applied to the selected
computers. Any resulting activities require explicit administrator
approval.

Adding a package profile
------------------------

Landscape uses package profiles (also called meta packages) to make sure
the proper software is installed when you request packages. You can
think of a package profile as a package with no file contents, just
dependencies and conflicts. With that information, the package profile
can trigger the installation of other packages necessary for the
requested package to run, or trigger the removal of software that
conflicts with the requested package. These dependencies and conflicts
fall under the general category of constraints.

To manage package profiles, click the PROFILES menu entry under your
account and the Package profiles link. The Package profiles screen
displays a list of existing package profiles and a link that you can
click to add a new package profile.

**Figure 7.4.**

![image](./Chapter%A07.%A0Managing%20packages_files/managepackages4.png)

\
Click on that link to display the Create package profile screen:

**Figure 7.5.**

![image](./Chapter%A07.%A0Managing%20packages_files/managepackages5.png)

\
Here you enter a name for the package profile, a description (which
appears at the top of the package profile's information screen), the
access group to which the package profile should belong, and,
optionally, any package constraints - packages that this profile depends
on or conflicts with. The constraints drop-down lists lets you add
constraints in three ways: based on a computer's installed packages,
imported from a previously exported CSV file or the output of the **dpkg
--get-selections** command, or manually added. Use the first option if
you want to replicate one computer to another, as it makes all currently
installed packages that are on the selected computer dependencies of the
package profile you're creating. The second approach imports the
dependencies of a previously exported package profile. The manual
approach is suitable when you have few dependencies to add, all of which
you know.

When you save a package profile, behind the scenes Landscape creates a
Debian package with the specified dependencies and conflicts and gives
it a name and a version. Every time you change the package profile,
Landscape increments the version by one.

If Landscape finds computers on which the package profile should be
installed, it creates an activity to do so. That activity will run
unattended, except that you must provide explicit administrator approval
to remove any packages that the package profile wants to delete.

Exporting a package profile
---------------------------

You can export a package profile in order to use the same constraints
it's set up for with a new package profile. To export a package profile,
click the PROFILES menu entry under your account and the Package
profiles link. Tick the check box next to the packages you want to
export, then click Download as CSV.

Modifying a package profile
---------------------------

To modify a package profile, click the PROFILES menu entry under your
account and the Package profiles link, then click on the name of a
package profile in the list.

Deleting a package profile
--------------------------

To delete a package profile, click the PROFILES menu entry under your
account and then the Package profiles link. Tick the check box next to
the packages you want to delete, then click Remove. The package profile
is deleted immediately, with no prompt to confirm the action.

Repositories
------------

Packages are stored in repositories. A repository is simply a designated
location that stores packages. You can manage Landscape repositories
only via [the Landscape
API](https://landscape.canonical.com/static/doc/user-guide/ch09.html "Chapter 9. The Landscape API").




* * * * *

[Prev](https://landscape.canonical.com/static/doc/user-guide/ch07.html)
[Up](https://landscape.canonical.com/static/doc/user-guide/index.html)
[Next](https://landscape.canonical.com/static/doc/user-guide/ch09.html)

##Use cases
--------------------


You can use Landscape to perform many common system administration tasks
easily and automatically. Here are a few examples.

How do I upgrade all packages on a certain group of machines?
-------------------------------------------------------------

First, tag the machines you want to upgrade with a common tag, so you
can use the tag anytime you need to manage those computers as a group.
If, for instance, you want to upgrade all your desktop computers, you
might want to use "desktop" as a tag. Select your computers, then click
on COMPUTERS on the top menu, and under that INFO. In the box under
Tags:, enter the tag you want to use and click the Add button.

If you've already tagged the computers, click on COMPUTERS, then click
on the tag in the left column.

With your desktop computers selected, click on COMPUTERS, then PACKAGES.
Scroll to the bottom of the screen, where you'll see a Request upgrades
button. Click it to queue the upgrade tasks.

![image](./Chapter%A08.%A0Use%20cases_files/usecases1.png)

While the upgrade tasks are now in the queue, they will not be executed
until you approve them. To do so, next to Select:, click All, then click
on the Approve button at the bottom of the page.

How do I keep all of my file servers automatically up to date?
--------------------------------------------------------------

The best way is to use [upgrade
profiles](https://landscape.canonical.com/static/doc/user-guide/ch02.html#defineupgradeprofiles),
which rely on access groups.

If an access group for your file servers already exists, simply click on
its name. If not, you must create an access group for them. To do so,
click on your account, then on ACCESS GROUPS. Specify a name for your
new access group and click the Save button. You must then add computers
to the access group. To do that, click on COMPUTERS, then select all
your file servers by using a tag, if one exists, or a search, or by
ticking them individually. Once all the computers you want to add to the
access group are tagged, click on the INFO menu choice, scroll down to
the bottom section, choose the access group you want from the drop-down
list, then click the Update access group button.

![image](./Chapter%A08.%A0Use%20cases_files/accessgroups4.png)

Once you have all your file servers in an access group you can create an
upgrade profile for them. Click on your account, then PROFILES menu
following the Upgrade profiles link, and then on the Add upgrade profile
link. Enter a name for the new upgrade profile, choose the access group
you wish to associate with it, and specify the schedule on which the
upgrades should run, then click the Save button.

How do I keep Landscape from upgrading a certain package on one of my servers?
------------------------------------------------------------------------------

First find the package by clicking on COMPUTERS, then PACKAGES. Use the
search box at the top of the screen to find the package you want. Click
the triangle on the left of the listing line of the package you want to
hold, which expands the information for that package. Now click on the
icon to the left of the package name. A new icon with a lock replaces
the old one, indicating that this package is to be held during upgrades.
Scroll to the bottom of the page and click on the Apply Changes button.

![image](./Chapter%A08.%A0Use%20cases_files/usecases2.png)

How do I set up a custom graph?
-------------------------------

First select the computers whose information you want to see. One good
way to do so is to create a tag for that group of computers on my
computers. Suppose you want to monitor the size of the PostgreSQL
database on your database servers. Select the servers, then click on
COMPUTERS on the top menu, and INFO under that. In the box under Tags:,
enter a tag name, such as "db-server," and click the Add button. Next,
under your account, click on CUSTOM GRAPHS, then on the link to Add
custom graph. Enter a title, and in the \#! field, enter **/bin/sh** to
indicate a shell script. In the Code section, enter the commands
necessary to create the data for the graph. For this example, the
command might be:

~~~~ {.programlisting}
psql -tAc "select pg_database_size('postgres')"
~~~~

For Run as user, enter **postgres**.

Fill in the Y-axix title, then click the Save button at the bottom of
the page.

![image](./Chapter%A08.%A0Use%20cases_files/usecases3.png)

To view the graph, click on COMPUTERS, then MONITORING. You can select
the monitoring period from the drop-down box at the top of the window.

How do I ensure all computers with a given tag have a common list of packages installed?
----------------------------------------------------------------------------------------

Manage them via a [package
profile](https://landscape.canonical.com/static/doc/user-guide/ch07.html#definepp "Adding a package profile").


