Title: Installing MAAS 
Status: In progress
Notes:





#Installing the MAAS software

##Scope of this documentation

This document provides instructions on how to install the Metal As A Service (MAAS) software. It has been prepared alongside guides for installing Juju, OpenStack and Landscape as part of a production grade cloud environment. MAAS itself may be used in different ways and you can find documentation for this on the main MAAS website [MAAS docs]. For the purposes of this documentation, the following assumptions have been made:
* You have sufficient, appropriate node hardware 
* You will be using Juju to assign workloads to MAAS
* You will be configuring the cluster network to be controlled entirely by MAAS (i.e. DNS and DHCP)
* If you have a compatible power-management system, any additional hardware required is also installed(e.g. IPMI network).

## Introducing MAAS

Metal as a Service – MAAS – lets you treat physical servers like virtual machines in the cloud. Rather than having to manage each server individually, MAAS turns your bare metal into an elastic cloud-like resource.

What does that mean in practice? Tell MAAS about the machines you want it to manage and it will boot them, check the hardware’s okay, and have them waiting for when you need them. You can then pull nodes up, tear them down and redeploy them at will; just as you can with virtual machines in the cloud.

When you’re ready to deploy a service, MAAS gives Juju the nodes it needs to power that service. It’s as simple as that: no need to manually provision, check and, afterwards, clean-up. As your needs change, you can easily scale services up or down. Need more power for your Hadoop cluster for a few hours? Simply tear down one of your Nova compute nodes and redeploy it to Hadoop. When you’re done, it’s just as easy to give the node back to Nova.

MAAS is ideal where you want the flexibility of the cloud, and the hassle-free power of Juju charms, but you need to deploy to bare metal.

## Installing MAAS from the Cloud Archive

The Ubuntu Cloud Archive is a repository made especially to provide users with the most up to date, stable versions of MAAS, Juju and other tools. It is highly recommended to configure this repository and use it to keep your software up to date:

```
sudo add-apt-repository cloud-archive:tools
sudo apt-get update
```

There are several packages that comprise a MAAS install. These are:

maas-region-controller:
    Which comprises the 'control' part of the software, including the web-based user interface, the API server and the main database.
maas-cluster-controller:
    This includes the software required to manage a cluster of nodes, including managing DHCP and boot images.
maas-dns:
    This is a customised DNS service that MAAS can use locally to manage DNS for all the connected nodes.
mass-dhcp:
    As for DNS, there is a DHCP service to enable MAAS to correctly enlist nodes and assign IP addresses. The DHCP setup is critical for the correct PXE booting of nodes.

As a convenience, there is also a `maas` metapackage, which will install all these components


If you need to separate these services or want to deploy an additional cluster controller, you should install the corresponding packages individually (see [_the description of a typical setup_](https://www.filepicker.io/api/file/orientation.html#setup) for more background on how a typical hardware setup might be arranged).




### Installing the packages

The configuration for the MAAS controller will automatically run and pop up this config screen:

![]( install_cluster-config.png)

Here you will need to enter the hostname for where the region controller can be contacted. In many scenarios, you may be running the region controller (i.e. the web and API interface) from a different network address, for example where a server has several network interfaces.

Once the configuration scripts have run you should see this message telling you that the system is ready to use:

![]( install_controller-config.png)

The web server is started last, so you have to accept this message before the service is run and you can access the Web interface. Then there are just a few more setup steps [_Post-Install tasks_](https://www.filepicker.io/api/file/WMGTttJT6aaLnQrEkAPv?signature=a86d0c3b4e25dba2d34633bbdc6873d9d8e6ae3cecc4672f2219fa81ee478502&policy=eyJoYW5kbGUiOiJXTUdUdHRKVDZhYUxuUXJFa0FQdiIsImV4cGlyeSI6MTM5NTE3NDE2MSwiY2FsbCI6WyJyZWFkIl19#post-install)

The maas-dhcp and maas-dns packages should be installed by default. You can check whether they are installed with:

```
dpkg -l maas-dhcp maas-dns
```

If they are missing, then:

```
sudo apt-get install maas-dhcp maas-dns
```

And then proceed to the post-install setup below.

If you now use a web browser to connect to the region controller, you should see that MAAS is running, but there will also be some errors on the screen:

![]( install_web-init.png)

The on screen messages will tell you that there are no boot images present, and that you can't login because there is no admin user.

## Create a superuser account

Once MAAS is installed, you'll need to create an administrator account:

```
sudo maas createadmin --username=root --email=MYEMAIL@EXAMPLE.COM
```

Substitute your own email address in the command above. You may also use a different username for your administrator account, but "root" is a common convention and easy to remember. The command will prompt for a password to assign to the new user.

You can run this command again for any further administrator accounts you may wish to create, but you need at least one.

## Import the boot images

MAAS will check for and download new Ubuntu images once a week. However, you'll need to download them manually the first time. To do this you will need to connect to the MAAS API using the maas-cli tool. (see for details). Then you need to run the command:

```
maas-cli maas node-groups import-boot-images
```

(substitute in a different profile name for 'maas' if you have called yours something else) This will initiate downloading the required image files. Note that this may take some time depending on your network connection.


## Login to the server

To check that everything is working properly, you should try and login to the server now. Both the error messages should have gone (it can take a few minutes for the boot image files to register) and you can see that there are currently 0 nodes attached to this controller.

![]( install-login.png)
## Configure switches on the network

Some switches use Spanning-Tree Protocol (STP) to negotiate a loop-free path through a root bridge. While scanning, it can make each port wait up to 50 seconds before data is allowed to be sent on the port. This delay in turn can cause problems with some applications/protocols such as PXE, DHCP and DNS, of which MAAS makes extensive use.

To alleviate this problem, you should enable [Portfast](https://www.symantec.com/business/support/index?page=content&id=HOWTO6019) for Cisco switches or its equivalent on other vendor equipment, which enables the ports to come up almost immediately.

##Add an additional cluster

Whilst it is certainly possible to run MAAS with just one cluster controller for all the nodes, in the interests of easier maintenance, uprades and stability, it is desirable to have at least two operational clusters.

Each cluster needs a controller node. Install Ubuntu on this node and then follow a similar setup proceedure to install the cluster controller software:

```
sudo add-apt-repository cloud-archive:tools
sudo apt-get update
sudo apt-get install maas-cluster-controller
sudo apt-get install maas-dhcp
```

During the install process, a configuration window will appear. You merely need to type in the address of the MAAS controller API, like this:

![config-image.png]

## Configure Cluster Controller(s)

### Cluster acceptance
When you install your first cluster controller on the same system as the region controller, it will be automatically accepted by default (but not yet configured, see below). Any other cluster controllers you set up will show up in the user interface as “pending,” until you manually accept them into the MAAS.

To accept a cluster controller, click on the settings “cog” icon at the top right to visit the settings page:
![]settings.png
You can either click on “Accept all” or click on the edit icon to edit the cluster. After clicking on the edit icon, you will see this page:

![]cluster-edit.png
Here you can change the cluster’s name as it appears in the UI, its DNS zone, and its status. Accepting the cluster changes its status from “pending” to “accepted.”

Now that the cluster controller is accepted, you can configure one or more of its network interfaces to be managed by MAAS. This will enable the cluster controller to manage nodes attached to those networks. The next section explains how to do this and what choices are to be made.

### Configuration
MAAS automatically recognises the network interfaces on each cluster controller. Some of these will be connected to networks where you want to manage nodes. We recommend letting your cluster controller act as a DHCP server for these networks, by configuring those interfaces in the MAAS user interface.

As an example, we will configure the cluster controller to manage a network on interface eth0. Click on the edit icon for eth0, which takes us to this page:

![]cluster-interface-edit.png
Here you can select to what extent you want the cluster controller to manage the network:

DHCP only - this will run a DHCP server on your cluster
DHCP and DNS - this will run a DHCP server on the cluster and configure the DNS server included with the region controller so that it can be used to look up hosts on this network by name.
Note
You cannot have DNS management without DHCP management because MAAS relies on its own DHCP server’s leases file to work out the IP address of nodes in the cluster.
If you set the interface to be managed, you now need to provide all of the usual DHCP details in the input fields below. Once done, click “Save interface”. The cluster controller will now be able to boot nodes on this network.

!!! note:There is also an option to leave the network unmanaged. Use this for networks where you don’t want to manage any nodes. Or, if you do want to manage nodes but don’t want the cluster controller to serve DHCP, you may be able to get by without it. This is explained in Manual DHCP configuration.

!!! note: A single cluster controller can manage more than one network, each from a different network interface on the cluster-controller server. This may help you scale your cluster to larger numbers of nodes, or it may be a requirement of your network architecture.

## Enlisting nodes

Now that the MAAS controller is running, we need to make the nodes aware of MAAS and vice-versa. With MAAS controlling DHCP and nodes capable of PXE booting, this is straightforward

Automatic Discovery
With nodes set to boot from a PXE image, they will start, look for a DHCP server, receive the PXE boot details, boot the image, contact the MAAS server and shut down.

During this process, the MAAS server will be passed information about the node, including the architecture, MAC address and other details which will be stored in the database of nodes. You can accept and comission the nodes via the web interface. When the nodes have been accepted the selected series of Ubuntu will be installed.

To save time, you can also accept and commission all nodes from the commandline. This requires that you first login with the API key [1], which you can retrieve from the web interface:

```
maas-cli maas nodes accept-all
```

### Manually adding nodes

If your nodes are not capable of booting from PXE images, they can be manually registered with MAAS. On the Nodes screen:
![]add-node.png

Select 'Add node' and manually enter details about the node, including its MAC address. This is used to identify the node when it contacts the DHCP server.



## Preparing MAAS for Juju using Simplestreams

When Juju bootstraps a cloud, it needs two critical pieces of information:

1. The uuid of the image to use when starting new compute instances.
2. The URL from which to download the correct version of a tools tarball.

This necessary information is stored in a json metadata format called "simplestreams". For supported public cloud services such as Amazon Web Services, HP Cloud, Azure, etc, no action is required by the end user. However, those setting up a private cloud, or who want to change how things work (eg use a different Ubuntu image), can create their own metadata, after understanding a bit about how it works.

The simplestreams format is used to describe related items in a structural fashion.( [See the Launchpad project lp:simplestreams for more details on implementation](https://launchpad.net/simplestreams)). Below we will discuss how Juju determines which metadata to use, and how to create your own images and tools and have Juju use them instead of the defaults.

### Basic Workflow

Whether images or tools, Juju uses a search path to try and find suitable metadata. The path components (in order of lookup) are:

1. User supplied location (specified by tools-metadata-url or image-metadata-url config settings).
2. The environment's cloud storage.
3. Provider specific locations (eg keystone endpoint if on Openstack).
4. A web location with metadata for supported public clouds (https://streams.canonical.com).

Metadata may be inline signed, or unsigned. We indicate a metadata file is signed by using the '.sjson' extension. Each location in the path is first searched for signed metadata, and if none is found, unsigned metadata is attempted before moving onto the next path location.

Juju ships with public keys used to validate the integrity of image and tools metadata obtained from https://streams.canonical.com. So out of the box, Juju will "Just Work" with any supported public cloud, using signed metadata. Setting up metadata for a private (eg Openstack) cloud requires metadata to be generated using tools which ship with Juju.

### Image Metadata Contents

Image metadata uses a simplestreams content type of "image-ids". The product id is formed as follows:

com.ubuntu.cloud:server:<series_version>:<arch> For Example:
com.ubuntu.cloud:server:14.04:amd64 Non-released images (eg beta, daily etc) have product ids like:
com.ubuntu.cloud.daily:server:13.10:amd64

The metadata index and product files are required to be in the following directory tree (relative to the URL associated with each path component):

<path_url> |-streams |-v1 |-index.(s)json |-product-foo.(s)json |-product-bar.(s)json

The index file must be called "index.(s)json" (sjson for signed). The various product files are named according to the Path values contained in the index file.

Tools metadata uses a simplestreams content type of "content-download". The product id is formed as follows:

"com.ubuntu.juju:<series_version>:<arch>"

For example:

"com.ubuntu.juju:12.04:amd64"

The metadata index and product files are required to be in the following directory tree (relative to the URL associated with each path component). In addition, tools tarballs which Juju needs to download are also expected.

|-streams | |-v1 | |-index.(s)json | |-product-foo.(s)json | |-product-bar.(s)json | |-releases |-tools-abc.tar.gz |-tools-def.tar.gz |-tools-xyz.tar.gz

The index file must be called "index.(s)json" (sjson for signed). The product file and tools tarball name(s) match whatever is in the index/product files.

### Configuration

For supported public clouds, no extra configuration is required; things work out-of-the-box. However, for testing purposes, or for non-supported cloud deployments, Juju needs to know where to find the tools and which image to run. Even for supported public clouds where all required metadata is available, the user can put their own metadata in the search path to override what is provided by the cloud.

#### User specified URLs

These are initially specified in the environments.yaml file (and then subsequently copied to the jenv file when the environment is bootstrapped). For images, use "image-metadata-url"; for tools, use "tools-metadata-url". The URLs can point to a world readable container/bucket in the cloud, an address served by a http server, or even a shared directory which is accessible by all node instances running in the cloud.

Assume an Apache http server with base URL `https://juju-metadata` , providing access to information at `<base>/images` and `<base>/tools` . The Juju environment yaml file could have the following entries (one or both):

tools-metadata-url: https://juju-metadata/tools image-metadata-url: https://juju-metadata/images

The required files in each location is as per the directory layout described earlier. For a shared directory, use a URL of the form "file:///sharedpath".

#### Cloud storage

If no matching metadata is found in the user specified URL, environment's cloud storage is searched. No user configuration is required here - all Juju environments are set up with cloud storage which is used to store state information, charms etc. Cloud storage setup is provider dependent; for Amazon and Openstack clouds, the storage is defined by the "control-bucket" value, for Azure, the "storage-account-name" value is relevant.

The (optional) directory structure inside the cloud storage is as follows:

|-tools | |-streams | |-v1 | |-releases | |-images |-streams |-v1

Of course, if only custom image metadata is required, the tools directory will not be required, and vice versa.

Note that if juju bootstrap is run with the `--upload-tools` option, the tools and metadata are placed according to the above structure. That's why the tools are then available for Juju to use.

#### Provider specific storage

Providers may allow additional locations to search for metadata and tools. For OpenStack, Keystone endpoints may be created by the cloud administrator. These are defined as follows:

juju-tools the &LT;path_url&GT; value as described above in Tools Metadata Contentsproduct-streams the &LT;path_url&GT; value as described above in Image Metadata Contents

Other providers may similarly be able to specify locations, though the implementation will vary.

This is the default location used to search for image and tools metadata and is used if no matches are found earlier in any of the above locations. No user configuration is required.

There are two main issues when deploying a private cloud:

1. Image ids will be specific to the cloud.
2. Often, outside internet access is blocked

Issue 1 means that image id metadata needs to be generated and made available.

Issue 2 means that tools need to be mirrored locally to make them accessible.

Juju tools exist to help with generating and validating image and tools metadata. For tools, it is often easiest to just mirror `https://streams.canonical.com/tools` . However image metadata cannot be simply mirrored because the image ids are taken from the cloud storage provider, so this needs to be generated and validated using the commands described below.

The available Juju metadata tools can be seen by using the help command:

juju help metadata

The overall workflow is:

- Generate image metadata
- Copy image metadata to somewhere in the metadata search path
- Optionally, mirror tools to somewhere in the metadata search path
- Optionally, configure tools-metadata-url and/or image-metadata-url

#### Image metadata

Generate image metadata using

juju metadata generate-image -d <metadata_dir>

As a minimum, the above command needs to know the image id to use and a directory in which to write the files.

Other required parameters like region, series, architecture etc. are taken from the current Juju environment (or an environment specified with the -e option). These parameters can also be overridden on the command line.

The image metadata command can be run multiple times with different regions, series, architecture, and it will keep adding to the metadata files. Once all required image ids have been added, the index and product json files can be uploaded to a location in the Juju metadata search path. As per the Configuration section, this may be somewhere specified by the `image-metadata-url` setting or the cloud's storage etc.

Examples:

1. image-metadata-url

- upload contents of to `http://somelocation`
- set image-metadata-url to `http://somelocation/images`

2. Cloud storage

If run without parameters, the validation command will take all required details from the current Juju environment (or as specified by -e) and output the image id it would use to spin up an instance. Alternatively, series, region, architecture etc. can be specified on the command line to override the values in the environment config.
#### Tools metadata

Generally, tools and related metadata are mirrored from `https://streams.canonical.com/tools` . However, it is possible to manually generate metadata for a custom built tools tarball.

First, create a tarball of the relevant tools and place in a directory structured like this:

<tools_dir>/tools/releases/

Now generate relevant metadata for the tools by running the command:

juju generate-tools -d <tools_dir>

Finally, the contents of can be uploaded to a location in the Juju metadata search path. As per the Configuration section, this may be somewhere specified by the tools-metadata-url setting or the cloud's storage path settings etc.

Examples:

1. tools-metadata-url

- upload contents of the tools dir to `http://somelocation`
- set tools-metadata-url to `http://somelocation/tools`

2. Cloud storage

upload contents of directly to environment's cloud storage

As with image metadata, the validation command is used to ensure tools are available for Juju to use:

juju metadata validate-tools

The same comments apply. Run the validation tool without parameters to use details from the Juju environment, or override values as required on the command line. See `juju help metadata validate-tools` for more details.

##Appendix I - Using Tags
##Appendix II - Using the MAAS CLI
As well as the web interface, many tasks can be performed by accessing the MAAS API directly through the maas-cli command. This section details how to login with this tool and perform some common operations.

###Logging in
Before the API will accept any commands from maas-cli, you must first login. To do this, you need the API key which can be found in the user interface.

Login to the web interface on your MAAS. Click on the username in the top right corner and select ‘Preferences’ from the menu which appears.

![]maascli-prefs.png
A new page will load...

![]maascli-key.png
The very first item is a list of MAAS keys. One will have already been generated when the system was installed. It’s easiest to just select all the text, copy the key (it’s quite long!) and then paste it into the commandline. The format of the login command is:

```
 maas-cli login <profile-name> <hostname> <key>
```

The profile created is an easy way of associating your credentials with any subsequent call to the API. So an example login might look like this:

```
maas-cli login maas http://10.98.0.13/MAAS/api/1.0
AWSCRMzqMNy:jjk...5e1FenoP82Qm5te2
```
which creates the profile ‘maas’ and registers it with the given key at the specified API endpoint. If you omit the credentials, they will be prompted for in the console. It is also possible to use a hyphen, ‘-‘ in place of the credentials. In this case a single line will be read from stdin, stripped of any whitespace and used as the credentials, which can be useful if you are devolping scripts for specific tasks. If an empty string is passed instead of the credentials, the profile will be logged in anonymously (and consequently some of the API calls will not be available)

### maas-cli commands
The maas-cli command exposes the whole API, so you can do anything you actually can do with MAAS using this command. This leaves us with a vast number of options, which are more fully expressed in the complete [2][MAAS Documentation]

list:
    lists the details [name url auth-key] of all the currently logged-in profiles.

login <profile> <url> <key>:
    Logs in to the MAAS controller API at the given URL, using the key provided and
    associates this connection with the given profile name.

logout <profile>:
    Logs out from the given profile, flushing the stored credentials.

refresh:
    Refreshes the API descriptions of all the current logged in profiles. This may become necessary for example when upgrading the maas packages to ensure the command-line options match with the API.

### Useful examples

Displays current status of nodes in the commissioning phase:
```
maas cli maas nodes check-commissioning
```

Accept and commission all discovered nodes:
```
maas-cli maas nodes accept-all
```

List all known nodes:
```
maas-cli maas nodes list
```

Filter the list using specific key/value pairs:
```
maas-cli maas nodes list architecture="i386/generic"
```

Set the power parameters for an ipmi enabled node:
```
maas-cli maas node update <system_id> \
  power_type="ipmi" \
  power_parameters_power_address=192.168.22.33 \
  power_parameters_power_user=root \
  power_parameters_power_pass=ubuntu;
```
## Appendix III - Physical Zones

To help you maximise fault-tolerance and performance of the services you deploy, MAAS administrators can define _physical zones_ (or just _zones_ for short), and assign nodes to them. When a user requests a node, they can ask for one that is in a specific zone, or one that is not in a specific zone.

It's up to you as an administrator to decide what a physical zone should represent: it could be a server rack, a room, a data centre, machines attached to the same UPS, or a portion of your network. Zones are most useful when they represent portions of your infrastructure. But you could also use them simply to keep track of where your systems are located.

Each node is in one and only one physical zone. Each MAAS instance ships with a default zone to which nodes are attached by default. If you do not need this feature, you can simply pretend it does not exist.

### Applications

Since you run your own MAAS, its physical zones give you more flexibility than those of a third-party hosted cloud service. That means that you get to design your zones and define what they mean. Below are some examples of how physical zones can help you get the most out of your MAAS.

### Creating a Zone

Only administrators can create and manage zones. To create a physical zone in the web user interface, log in as an administrator and browse to the "Zones" section in the top bar. This will takes you to the zones listing page. At the bottom of the page is a button for creating a new zone:

![]add-zone.png

Or to do it in the [_region-controller API_][#region-controller-api], POST your zone definition to the _"zones"_ endpoint.

### Assigning Nodes to a Zone

Once you have created one or more physical zones, you can set nodes' zones from the nodes listing page in the UI. Select the nodes for which you wish to set a zone, and choose "Set physical zone" from the "Bulk action" dropdown list near the top. A second dropdown list will appear, to let you select which zone you wish to set. Leave it blank to clear nodes' physical zones. Clicking "Go" will apply the change to the selected nodes.

You can also set an individual node's zone on its "Edit node" page. Both ways are available in the API as well: edit an individual node through a request to the node's URI, or set the zone on multiple nodes at once by calling the operation on the endpoint.

### Allocating a Node in a Zone

To deploy in a particular zone, call the method in the [_region-controller API_][#region-controller-api] as before, but pass the parameter with the name of the zone. The method will allocate a node in that zone, or fail with an HTTP 409 ("conflict") error if the zone has no nodes available that match your request.

Alternatively, you may want to request a node that is _not_ in a particular zone, or one that is not in any of several zones. To do that, specify the parameter to . This parameter takes a list of zone names; the allocated node will not be in any of them. Again, if that leaves no nodes available that match your request, the call will return a "conflict" error.

It is possible, though not usually useful, to combine the and parameters. If your choice for is also present in , no node will ever match your request. Or if it's not, then the values will not affect the result of the call at all.

